
// Serial LCD Interface
// Zeichen mit UART empfangen und auf einem LCD Display ausgeben
//
// ATmega8, 16MHz
// AVR Studio 4, Optimierungsgrad: O2
// erstellt am 3.08.2012
// www.alex-handfest.de


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "lcd.h"
#include "uart.h"



// gespeicherte Variablen im EEPROM
unsigned char eeFoooc1a EEMEM = 200;
unsigned char eeFoooc1b EEMEM = 60;
unsigned char eeFoospar EEMEM = 40;
unsigned char eeFoospst EEMEM = 1;
unsigned char eeFoodimm EEMEM = 1;
unsigned char eeFooakti EEMEM = 30;
unsigned char eeFooflus EEMEM = 1;
unsigned char eeFoozeil EEMEM = 2;
		float eeFoouart EEMEM = 19200;
unsigned char eeFoostart1[40] EEMEM = {"    HAL 9000"};
unsigned char eeFoostart2[40] EEMEM = {'\0'};
unsigned char eeFoostart3[40] EEMEM = {'\0'};
unsigned char eeFoostart4[40] EEMEM = {'\0'};


// Variablen UART
extern volatile unsigned char uart_cr;
extern volatile unsigned char uart_lf;
extern volatile unsigned long uart_baut;
extern volatile unsigned char uart_recv_buf[41];

// Variablen LCD
char volllicht = 0;
char sparlicht = 0;
char dimmlicht = 0;
char sparcount = 0;
char sparstatu = 0;
char aktivzeit = 0;
char akticount = 0;
char textfluss = 0;
char zeilennum = 0;
char zeilenpos = 0;

// Textspeicher
char text1[41] = {'\0'};
char text2[41] = {'\0'};
char text3[41] = {'\0'};
char text4[41] = {'\0'};




// =====================================================================================


void textloeschen(void){

	for(int i = 0; i<40; i++) text1[i]='\0';		// msste auch gehen, wenn nur der
	for(int i = 0; i<40; i++) text2[i]='\0';		// erste Wert auf \0 gesetzt wird 
	for(int i = 0; i<40; i++) text3[i]='\0';
	for(int i = 0; i<40; i++) text4[i]='\0';
}


void textschreiben(void){

	lcd_clear();
	lcd_string(text1);
	lcd_setcursor(0,2);
	lcd_string(text2);
	lcd_setcursor(0,3);
	lcd_string(text3);
	lcd_setcursor(0,4);
	lcd_string(text4);
}


void sparreset(void){

	OCR1A = volllicht;
	akticount = 0;
	if (dimmlicht >= 1) dimmlicht = 2;
}


// =====================================================================================


void initialisieren(void){

	// Variablen einlesen
	volllicht = eeprom_read_byte (&eeFoooc1a);
	OCR1B     = eeprom_read_byte (&eeFoooc1b);
	sparlicht = eeprom_read_byte (&eeFoospar);
	sparstatu = eeprom_read_byte (&eeFoospst);
	dimmlicht = eeprom_read_byte (&eeFoodimm);
	aktivzeit = eeprom_read_byte (&eeFooakti);
	textfluss = eeprom_read_byte (&eeFooflus);
	zeilennum = eeprom_read_byte (&eeFoozeil);
	uart_baut = eeprom_read_float(&eeFoouart);
	eeprom_read_block (text1, eeFoostart1, 40);
	eeprom_read_block (text2, eeFoostart2, 40);
	eeprom_read_block (text3, eeFoostart3, 40);
	eeprom_read_block (text4, eeFoostart4, 40);
	

	// Initialisierung des LCD
	lcd_init();

	// Initialisierung des UART
	uart_init();

	//Globale Interrupts einschalten
	sei();


	// Initialisierung von Beleuchtung und Kontrast
	// Timer1 init
	TCCR1B  = (0<<CS02)   | (0<<CS01)  | (1<<CS00);	// Setzen des Prescaler auf 8
	TCCR1A  = (0<<WGM11)  | (1<<WGM10);				// Mode1: PWM Phase Correct 8-Bit
	TCCR1A |= (1<<COM1A1) | (1<<COM1B1);			// nicht invertierter PWM

	// PWM Ausgnge
	DDRB |= (1 << DDB1) | (1 << DDB2);

	OCR1A = volllicht;

	
	// Initialisierung des Energiesparers
	// Timer0 init
	TCCR0 = (1<<CS02) | (0<<CS01) | (1<<CS00);   	//Setzen des Prescaler auf 1024
	TIMSK |= (1<<TOIE0);
	// Interrupt Aktion alle (16000000/1024) / 256 = 61,04 Hz -> 0,016384s


	// Startbildschirm
	textschreiben();
	textloeschen();
	lcd_home();

	sparreset();
}


// =====================================================================================





int main(void){

	initialisieren();

	while(1){

		//UART empfangen und auf dem LCD ausgeben:		
		if (uart_cr == 1){

			sparreset();

			if(textfluss == 1){				// alten String nach unten schieben

				for(int i = 0; i<40; i++) text4[i] = text3[i];
				for(int i = 0; i<40; i++) text3[i] = text2[i];
				for(int i = 0; i<40; i++) text2[i] = text1[i];
				for(int i = 0; i<40; i++) text1[i] = uart_recv_buf[i];
				
				textschreiben();
			}

			else if(textfluss == 2){		// alten String nach oben schieben

				if (zeilennum == 2){

					for(int i = 0; i<40; i++) text1[i] = text2[i];
					for(int i = 0; i<40; i++) text2[i] = uart_recv_buf[i];
				}
				else {
					for(int i = 0; i<40; i++) text1[i] = text2[i];
					for(int i = 0; i<40; i++) text2[i] = text3[i];
					for(int i = 0; i<40; i++) text3[i] = text4[i];
					for(int i = 0; i<40; i++) text4[i] = uart_recv_buf[i];
				}

				textschreiben();
			}

			else if(textfluss == 3){		// nur Enter

				lcd_setcursor(0,zeilenpos);

				zeilenpos++;				
				if (zeilenpos > zeilennum) zeilenpos = 1;

				lcd_string(uart_recv_buf);
			}
			
			else lcd_string(uart_recv_buf);
			
			uart_done();		
		}






		//UART empfangen und Befehle ausfhren:		
		else if (uart_lf == 1){

			sparreset();

			// ----- Beleuchtung / backlight  ---------------------------------------------
			if (uart_recv_buf[0] == 'B'){

				if (uart_recv_buf[1] == '1') volllicht = 255;
				if (uart_recv_buf[1] == '0') volllicht = 0;
				if (uart_recv_buf[1] == '=') volllicht = uart_getnumber(2);
				OCR1A = volllicht;
			}

			// ----- Kontrast / contrast  -------------------------------------------------
			else if (uart_recv_buf[0] == 'C'){

				if (uart_recv_buf[1] == '1') OCR1B = 255;
				if (uart_recv_buf[1] == '0') OCR1B = 0;
				if (uart_recv_buf[1] == '=') OCR1B = uart_getnumber(2);
			}

			// ----- alles lschen / delete all  ------------------------------------------
			else if (uart_recv_buf[0] == 'D'){

				lcd_clear();
				textloeschen();
			}

			// ----- Energiesparer / energysaver  -----------------------------------------
			else if (uart_recv_buf[0] == 'E'){

				lcd_clear();
				textloeschen();

				if (uart_recv_buf[1] == '1'){

					TIMSK |= (1<<TOIE0);
					sparstatu = 1;

					lcd_string("* Energiesparer");
					lcd_setcursor(0,2);
					lcd_string("* an");
				}

				if (uart_recv_buf[1] == '0'){

					TIMSK = 0x00;
					sparstatu = 0;

					lcd_string("* Energiesparer");
					lcd_setcursor(0,2);
					lcd_string("* aus");
				}

				if (uart_recv_buf[1] == 'D'){

					lcd_string("* Energiesparer");
					lcd_setcursor(0,2);

					if (uart_recv_buf[2] == '1'){

						dimmlicht = 2;		
						lcd_string("* gedimmt aus");
					}

					if (uart_recv_buf[2] == '0'){

						dimmlicht = 0;
						lcd_string("* sofot aus");
					}
				}

				if (uart_recv_buf[1] == 'T' && uart_recv_buf[2] == '='){

					aktivzeit = uart_getnumber(3);

					uart_recv_buf[0] = '*';
					uart_recv_buf[1] = 32;
					uart_recv_buf[2] = 32;
					lcd_string("* Zeit E-Spar:");
					lcd_setcursor(0,2);
					lcd_string(uart_recv_buf);
					lcd_string(" Sekunden");
				}

				if (uart_recv_buf[1] == 'B' && uart_recv_buf[2] == '='){

					sparlicht = uart_getnumber(3);

					uart_recv_buf[0] = '*';
					uart_recv_buf[1] = 32;
					uart_recv_buf[2] = 32;
					lcd_string("* Licht E-Spar:");
					lcd_setcursor(0,2);
					lcd_string(uart_recv_buf);
				}
				
			}

			// ----- Textfluss / textflow  ------------------------------------------------
			else if (uart_recv_buf[0] == 'F'){

				lcd_clear();
				textloeschen();

				if (uart_recv_buf[1] == '0'){
					
					textfluss = 0;
					lcd_string("* Textfluss aus");
				}

				if (uart_recv_buf[1] == '1'){
					
					textfluss = 1;
					lcd_string("* Textfluss 1");
					lcd_setcursor(0,2);
					lcd_string("* neu oben");
				}
				if (uart_recv_buf[1] == '2'){
					
					textfluss = 2;
					lcd_string("* Textfluss 2");
					lcd_setcursor(0,2);
					lcd_string("* neu unten");
				}
				if (uart_recv_buf[1] == '3'){
					
					textfluss = 3;
					zeilenpos = 0;
					lcd_string("* Textfluss 3");
					lcd_setcursor(0,2);
					lcd_string("* nur Enter");
				}
			}

			// ----- Zeilenzahl / lines  --------------------------------------------------
			else if (uart_recv_buf[0] == 'L' && uart_recv_buf[1] == '='){

				zeilennum = uart_getnumber(2);

				uart_recv_buf[0] = '*';
				uart_recv_buf[1] = 32;
				lcd_clear();
				textloeschen();
				lcd_string("* Anzahl Zeilen:");
				lcd_setcursor(0,2);
				lcd_string(uart_recv_buf);
			}

			// ----- Nummer / number  ----------------------------------------------------
			else if (uart_recv_buf[0] == 'N'){

				char num_buf[4]  = {'\0'};
				sprintf(num_buf, "%d", uart_recv_buf[1]);
				lcd_string(num_buf);
			}

			// ----- Zeiger / pointer  ----------------------------------------------------
			else if (uart_recv_buf[0] == 'P'){

				if (uart_recv_buf[1] == 'H') lcd_home();
				if (uart_recv_buf[1] == '1') lcd_setcursor(uart_getnumber(2),1);
				if (uart_recv_buf[1] == '2') lcd_setcursor(uart_getnumber(2),2);
				if (uart_recv_buf[1] == '3') lcd_setcursor(uart_getnumber(2),3);
				if (uart_recv_buf[1] == '4') lcd_setcursor(uart_getnumber(2),4);
			}

			// ----- Reset / reset  -------------------------------------------------------
			else if (uart_recv_buf[0] == 'R'){

				lcd_clear();
				lcd_string("reset...");
				textloeschen();

				eeprom_write_byte (&eeFoooc1a, 200);
				eeprom_write_byte (&eeFoooc1b, 60);
				eeprom_write_byte (&eeFoospar, 40);
				eeprom_write_byte (&eeFoospst, 1);
				eeprom_write_byte (&eeFoodimm, 1);
				eeprom_write_byte (&eeFooakti, 30);
				eeprom_write_byte (&eeFooflus, 1);
				eeprom_write_byte (&eeFoozeil, 2);
				eeprom_write_float(&eeFoouart, 19200);

				text1[0] = 32;
				text1[1] = 32;
				text1[2] = 32;
				text1[3] = 32;
				text1[4] = 'H';
				text1[5] = 'A';
				text1[6] = 'L';
				text1[7] = 32;
				text1[8] = '9';
				text1[9] = '0';
				text1[10] = '0';
				text1[11] = '0';

				eeprom_write_block (text1, eeFoostart1, 40);
				eeprom_write_block (text2, eeFoostart2, 40);
				eeprom_write_block (text3, eeFoostart3, 40);
				eeprom_write_block (text4, eeFoostart4, 40);

				textloeschen();
				lcd_clear();
				initialisieren();	
			}

			// ----- speichern / save  ----------------------------------------------------
			else if (uart_recv_buf[0] == 'S'){

				lcd_clear();
				lcd_string("saving...");

				if (uart_recv_buf[1] == 'V'){

					eeprom_write_byte (&eeFoooc1a, volllicht);
					eeprom_write_byte (&eeFoooc1b, OCR1B);
					eeprom_write_byte (&eeFoospar, sparlicht);
					eeprom_write_byte (&eeFoospst, sparstatu);
					eeprom_write_byte (&eeFoodimm, dimmlicht);
					eeprom_write_byte (&eeFooakti, aktivzeit);
					eeprom_write_byte (&eeFooflus, textfluss);
					eeprom_write_byte (&eeFoozeil, zeilennum);
					eeprom_write_float(&eeFoouart, uart_baut);

					lcd_clear();
					textloeschen();
					lcd_string("* alle Werte");
					lcd_setcursor(0,2);
					lcd_string("* gespeichert");
				}

				if (uart_recv_buf[1] == 'T'){

					eeprom_write_block (text1, eeFoostart1, 40);
					eeprom_write_block (text2, eeFoostart2, 40);
					eeprom_write_block (text3, eeFoostart3, 40);
					eeprom_write_block (text4, eeFoostart4, 40);

					lcd_clear();
					textloeschen();
					lcd_string("* Text als Start");
					lcd_setcursor(0,2);
					lcd_string("* gespeichert");
				}
			}

			// ----- UART Baud / UART Baud  -----------------------------------------------
			else if (uart_recv_buf[0] == 'U' && uart_recv_buf[1] == '='){

				uart_baut = uart_getnumber(2);
				uart_init();

				uart_recv_buf[0] = '*';
				uart_recv_buf[1] = 32;
				lcd_clear();
				textloeschen();
				lcd_string("* UART BAUT neu:");
				lcd_setcursor(0,2);
				lcd_string(uart_recv_buf);
			}


			uart_done();
		}

	}
	return 0;
}




// =======================================================================================
// --------- Time Overflow alle 0,016384s --------

ISR(TIMER0_OVF_vect){

	if (sparcount == 61){					// jede Sekunde

		if (akticount == aktivzeit){		// Aktivierungszeit

			if (dimmlicht >= 2) dimmlicht = 3;
			else OCR1A = sparlicht;

			akticount = 0;
		}

		akticount++;
		sparcount = 0;

	}

	if (dimmlicht == 3){

		OCR1A = OCR1A - 1;
		if (OCR1A <= sparlicht) dimmlicht = 1;
	}

	sparcount++;
}

